import { Component, h, State, Element, Listen, Prop, Watch } from '@stencil/core';
import { getSize } from '../../utils/size';
import { getLangData, useLanguage } from '../../lib/lang';
import { defaultIconPack, getIconPack } from '../../lib/icons';
import { defaultConfig } from '../../lib/default-ui-config';
import { Render } from '../../lib/render';
import { provideDyteDesignSystem } from '../../index';
import { getUserPreferences } from '../../utils/user-prefs';
import { generateConfig } from '../../utils/config';
import storeState from '../../lib/store';
import { isV2Theme } from '../../utils/versionTwoApis';
/**
 * A single component which renders an entire meeting UI.
 *
 * It loads your preset and renders the UI based on it.
 * With this component, you don't have to handle all the states,
 * dialogs and other smaller bits of managing the application.
 */
export class DyteMeeting {
  constructor() {
    this.roomJoinedListener = () => {
      this.setStates({ meeting: 'joined' });
      storeState.meeting = 'joined';
    };
    this.roomLeftListener = ({ state }) => {
      const states = this.states || storeState;
      if ((states === null || states === void 0 ? void 0 : states.roomLeftState) === 'disconnected') {
        this.setStates({ meeting: 'ended' });
        storeState.meeting = 'ended';
        return;
      }
      this.setStates({ meeting: 'ended', roomLeftState: state });
      storeState.meeting = 'ended';
      storeState.roomLeftState = state;
    };
    this.mediaPermissionErrorListener = ({ kind }) => {
      if (['audio', 'video'].includes(kind)) {
        this.setStates({ activePermissionsMessage: true });
        storeState.activePermissionsMessage = true;
      }
    };
    /** Whether to load config from preset */
    this.loadConfigFromPreset = true;
    /** Whether to apply the design system on the document root from config */
    this.applyDesignSystem = true;
    /** Fill type */
    this.mode = 'fixed';
    /** Language */
    this.language = 'en';
    /** UI Config */
    this.config = defaultConfig;
    this.states = {
      meeting: 'idle',
      prefs: {
        mirrorVideo: true,
        muteNotificationSounds: false,
        autoScroll: true,
      },
    };
    this.iconPack = defaultIconPack;
    this.t = useLanguage();
  }
  connectedCallback() {
    var _a;
    this.resizeObserver = new ResizeObserver(() => this.handleResize());
    this.resizeObserver.observe(this.host);
    if (this.applyDesignSystem &&
      ((_a = this.config) === null || _a === void 0 ? void 0 : _a.designTokens) != null &&
      typeof document !== 'undefined') {
      provideDyteDesignSystem(document.documentElement, this.config.designTokens);
    }
    this.meetingChanged(this.meeting);
    this.iconPackUrlChanged(this.iconPackUrl);
    this.languageChanged(this.language);
    this.initializePreferences();
  }
  disconnectedCallback() {
    var _a, _b;
    this.resizeObserver.disconnect();
    (_a = this.meeting) === null || _a === void 0 ? void 0 : _a.self.removeListener('roomJoined', this.roomJoinedListener);
    (_b = this.meeting) === null || _b === void 0 ? void 0 : _b.self.removeListener('roomLeft', this.roomLeftListener);
  }
  meetingChanged(meeting) {
    var _a;
    if (meeting != null) {
      if (this.loadConfigFromPreset && meeting.self.suggestedTheme != null) {
        const theme = meeting.self.suggestedTheme;
        const { config, data } = isV2Theme(theme)
          ? { config: defaultConfig, data: { showSetupScreen: true } }
          : generateConfig(theme);
        if (this.config === defaultConfig) {
          // only override the config if the object is same as defaultConfig
          // which means it's a different object passed via prop
          this.config = config;
        }
        if (this.showSetupScreen == null) {
          // only override this value if the prop isn't passed
          this.showSetupScreen = data.showSetupScreen;
        }
      }
      meeting.self.addListener('roomJoined', this.roomJoinedListener);
      meeting.self.addListener('roomLeft', this.roomLeftListener);
      meeting.self.addListener('mediaPermissionError', this.mediaPermissionErrorListener);
      if (this.applyDesignSystem &&
        ((_a = this.config) === null || _a === void 0 ? void 0 : _a.designTokens) != null &&
        typeof document !== 'undefined') {
        provideDyteDesignSystem(document.documentElement, this.config.designTokens);
      }
      if (this.showSetupScreen) {
        this.states = Object.assign(Object.assign({}, this.states), { meeting: 'setup' });
        storeState.meeting = 'setup';
      }
      else {
        // join directly to the meeting
        meeting.joinRoom();
      }
    }
  }
  async iconPackUrlChanged(url) {
    this.iconPack = await getIconPack(url);
  }
  async languageChanged(lang) {
    this.t = useLanguage(await getLangData(lang));
  }
  listenState(e) {
    e.stopPropagation();
    this.setStates(e.detail);
  }
  initializePreferences() {
    const prefs = getUserPreferences();
    this.setStates({ prefs });
    storeState.prefs = prefs;
  }
  handleResize() {
    this.size = getSize(this.host.clientWidth);
  }
  setStates(states) {
    this.states = Object.assign(Object.assign({}, this.states), states);
  }
  render() {
    const defaults = {
      meeting: this.meeting,
      size: this.size,
      states: this.states || storeState,
      config: this.config,
      iconPack: this.iconPack,
      t: this.t,
    };
    return h(Render, { element: "dyte-meeting", defaults: defaults, asHost: true });
  }
  static get is() { return "dyte-meeting"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["dyte-meeting.css"]
  }; }
  static get styleUrls() { return {
    "$": ["dyte-meeting.css"]
  }; }
  static get properties() { return {
    "loadConfigFromPreset": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Whether to load config from preset"
      },
      "attribute": "load-config-from-preset",
      "reflect": false,
      "defaultValue": "true"
    },
    "applyDesignSystem": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Whether to apply the design system on the document root from config"
      },
      "attribute": "apply-design-system",
      "reflect": false,
      "defaultValue": "true"
    },
    "mode": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "MeetingMode",
        "resolved": "\"fill\" | \"fixed\"",
        "references": {
          "MeetingMode": {
            "location": "local"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Fill type"
      },
      "attribute": "mode",
      "reflect": true,
      "defaultValue": "'fixed'"
    },
    "meeting": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Meeting",
        "resolved": "DyteClient",
        "references": {
          "Meeting": {
            "location": "import",
            "path": "../../types/dyte-client"
          }
        }
      },
      "required": true,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Meeting object"
      }
    },
    "showSetupScreen": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Where to show setup screen or not"
      },
      "attribute": "show-setup-screen",
      "reflect": false
    },
    "language": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Language",
        "resolved": "\"en\" | \"hi\"",
        "references": {
          "Language": {
            "location": "import",
            "path": "../../lib/lang"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Language"
      },
      "attribute": "language",
      "reflect": true,
      "defaultValue": "'en'"
    },
    "config": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "UIConfig",
        "resolved": "UIConfig",
        "references": {
          "UIConfig": {
            "location": "import",
            "path": "../../types/ui-config"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "UI Config"
      },
      "defaultValue": "defaultConfig"
    },
    "iconPackUrl": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Icon Pack URL"
      },
      "attribute": "icon-pack-url",
      "reflect": true
    },
    "size": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "Size",
        "resolved": "\"lg\" | \"md\" | \"sm\" | \"xl\"",
        "references": {
          "Size": {
            "location": "import",
            "path": "../../types/props"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Size"
      },
      "attribute": "size",
      "reflect": true
    }
  }; }
  static get states() { return {
    "states": {},
    "iconPack": {},
    "t": {}
  }; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "meeting",
      "methodName": "meetingChanged"
    }, {
      "propName": "iconPackUrl",
      "methodName": "iconPackUrlChanged"
    }, {
      "propName": "language",
      "methodName": "languageChanged"
    }]; }
  static get listeners() { return [{
      "name": "dyteStateUpdate",
      "method": "listenState",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
